Program
    Uses
        cobra2d,
        keyset,
        hagC2D,             // http://www.tophatstuff.co.uk/?p=42
        xml2gui,            // http://www.tophatstuff.co.uk/?p=42
        poxml               // http://www.tophatstuff.co.uk/?p=103

{ Simple example that loads a GUI from an XML file and grabs the handles to the elements }

Var
    bg: Element
    
    tmpForm: ^forms
    statusText: ^hTexts
    drawCanvas: ^canvases
    canvasPenColor: ^dropDowns
    canvasPenThickness: ^sliders
    canvasClear: ^buttons

Procedure Init()
{
    Sets up the window and the GUI
}
Begin

    // Window
    SetAppName("Editor Example")
    OpenScreen(640, 480, 32, FALSE, COB_SHOWBORDER)
    VSync(1)
    
    // Background
    bg = CreateSprite(640, 480)
    CLS(ToRGBA(220, 220, 200), bg)
    
    // Init GUI    
    HagBaseDir('hag\')                  // Optional - where to find the hag folder (defaults to 'hag\')
    HagInit(640, 480, 1)               // Required - width, height, [SpriteIndex] (set to 1 if you have a background sprite)
    HagEnableDebugLog(TRUE)             // Optional - debugging (see 'hag\log\' for log files)    
    HagLoadGuiTheme('XP')               // Required - the name of a theme in 'hag\themes\gui\'  
    
    // Load GUI from XML..
    Xml2Gui('main.xml')
    
    // Get handles to elements...
    
    // --- Toolbar
    tmpForm = GetFormByHook('toolbar')
    canvasPenColor = GetDropDownByHook(tmpForm, 'toolbar.color')
    canvasPenThickness = GetSliderByHook(tmpForm, 'toolbar.thickness')
    canvasClear = GetButtonByHook(tmpForm, 'toolbar.clear')
    
    // --- Canvas
    tmpForm = GetFormByHook('canvas')
    drawCanvas = GetCanvasByHook(tmpForm, 'canvas.drawCanvas')
    
    // --- Status bar
    tmpForm = GetFormByHook('statusBar')
    statusText = GetTextByHook(tmpForm, 'statusBar.statusText') 


End

Procedure BLine(x0: Integer, y0: Integer, x1: Integer, y1: Integer, color: Integer, canvas: Element, xthickness: Integer, ythickness: Integer)
Var
    steep: Boolean
    iTmp: Integer
    deltax, deltay: Integer
    error: Integer
    ystep: Integer
    x, y: Integer
    xthickOver2, ythickOver2: Integer
Begin
    // http://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm
    // Instead of plotting pixels, this algo is modified to plot
    // rects of size (xthickness, ythickness)

    steep = abs(y1 - y0) > abs(x1 - x0) 
 
    If steep then
        
        // swap(x0, y0)
        iTmp = x0
        x0 = y0
        y0 = iTmp
        
        // swap(x1, y1)
        iTmp = x1
        x1 = y1
        y1 = iTmp        
        
    Endif
        
     If x0 > x1 then

         // swap(x0, x1)
        iTmp = x0
        x0 = x1
        x1 = iTmp
         
         // swap(y0, y1)
        iTmp = y0
        y0 = y1
        y1 = iTmp
         
     Endif
     
     deltax = x1 - x0
     deltay = abs(y1 - y0)
     error = deltax / 2
     ystep = 0
     y = y0
     
     If y0 < y1 then ystep = 1 else ystep = -1
     
     For x = x0 to x1
     
        xthickOver2 = xthickness/2
        ythickOver2 = ythickness/2
     
        If steep then
            Rect(y-ythickOver2, x-xthickOver2, xthickness, ythickness, color, TRUE, canvas)
        Else
            Rect(x-xthickOver2, y-ythickOver2, xthickness, ythickness, color, TRUE, canvas)
        Endif
        
        error = error - deltay
        If error < 0 then
            y = y + ystep
            error = error + deltax
        Endif
    Next

End

Var
    lastTooltip: String = ""
    s: String
    t: Integer
    x, y, x2, y2: Integer
    
    color: Integer
    thickness: Integer
    lastx, lasty: Integer = -1

Begin
    Init()
    
    color = ToRGBA(0,0,0)
    thickness = 1
    
    // CLear the canvas
    CLS(ToRGBA(255,255,255), drawCanvas.e)
    
 
    While KeyHits(VK_ESCAPE) < 1
        
        // Updates the GUI automatically capturing the input needed
        // For finder control, see HagUpdateAll in the documentation
        HagUpdateAll_AutoKeys()
        
        // Updates the tooltip (hag_tooltip is an exported global from hag)
        If hag_tooltip <> lastTooltip then
            TextText(statusText, hag_tooltip)
            lastTooltip = hag_tooltip
        Endif
        
        // Update Elements...
        
        // --- Clear Canvas
        If ButtonClicked(canvasClear) then CLS(ToRGBA(255,255,255), drawCanvas.e)
        
        // --- Pen Color
        If DropDownUpdated(canvasPenColor) then
            s = Lower(canvasPenColor.txt)
            
            Conditions
                (s = 'black'):  color = ToRGBA(0,0,0)
                (s = 'white'):  color = ToRGBA(255,255,255)
                (s = 'red'):    color = ToRGBA(255,0,0)
                (s = 'green'):  color = ToRGBA(0,255,0)
                (s = 'blue'):   color = ToRGBA(0,0,255)
            EndConditions
        Endif
        
        // --- Pen Thickness
        If SliderUpdated(canvasPenThickness) then
            t = SliderGetDivision(canvasPenThickness) + 1
            thickness = t + 1
        Endif
        
        // --- Canvas
        If drawCanvas.hover then
            If MouseDown(0) then
                x = MouseX - drawCanvas.x
                y = MouseY - drawCanvas.y
                
                If lastx > -1 then
                    x2 = lastx
                    y2 = lasty
                Else
                    x2 = x
                    y2 = y 
                Endif
                
                BLine(x, y, x2, y2, color, drawCanvas.e, thickness, thickness)
                
                lastx = x
                lasty = y
            Else
                lastx = -1
                lasty = -1
            Endif
        Endif 
    
        Flip
        Pause(1)
    Wend
End